"""RSA鍵を使用した署名・検証
"""

from base64 import urlsafe_b64encode, urlsafe_b64decode
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5
from Crypto.Hash import SHA256


def get_authenticator():
    return Authenticator()


class Authenticator:
    def sign(self, message, str_key):
        """署名

        Args:
            message (str): 署名する文字列
            str_key (str): RSA秘密鍵
        
        Returns:
            str: 署名
        """
        # RSA秘密鍵
        key = RSA.importKey(str_key)
        # 署名、base64符号化
        signature = urlsafe_b64encode(self.__sign(message, key))
        # 文字列に変換
        return signature.decode('utf-8')

    def verify(self, message, sign, str_key):
        """検証

        Args:
            message (str): 検証する文字列
            sign (str): 署名
            str_key: RSA公開鍵
        
        Returns:
            bool: 対応する秘密鍵で署名されていた場合は True
        """
        # RSA公開鍵
        key = RSA.importKey(str_key)
        # base64復号化、検証
        return self.__verify(message, urlsafe_b64decode(sign), key)

    def __sign(self, message, private_key):
        # SHA256ダイジェスト値
        digest = SHA256.new()
        digest.update(message.encode('utf-8'))
        # RSA秘密鍵で署名
        signer = PKCS1_v1_5.new(private_key)
        return signer.sign(digest)

    def __verify(self, message, signature, pub_key):
        # SHA256ダイジェスト値
        digest = SHA256.new()
        digest.update(message.encode('utf-8'))
        # RSA公開鍵で検証
        signer = PKCS1_v1_5.new(pub_key)
        return signer.verify(digest, signature)
